#include "MainFrame.h"
#include "AboutDlg.h"
#include "OptionsDlg.h"
#include "StatsDlg.h"
#include "ResultDlg.h"
#include <fstream>
#include <wx/regex.h>

const char *SETTINGSFILE = "settings.sto";
const char *LOGFILE = "higlog.txt";

using namespace std;

BEGIN_EVENT_TABLE(PanelPainter, wxEvtHandler)
    EVT_PAINT(OnPaint)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
    EVT_MENU(menuNew, MainFrame::OnMenuNew)
    EVT_MENU(menuRestart, MainFrame::OnMenuRestart)
    EVT_MENU(menuOptions, MainFrame::OnMenuOptions)
    EVT_MENU(menuStats, MainFrame::OnMenuStats)
    EVT_MENU(menuExit, MainFrame::OnMenuExit)
    EVT_MENU(menuAbout, MainFrame::OnMenuAbout)
    EVT_TEXT_ENTER(txtEnter, MainFrame::OnTxtEnter)
    EVT_CLOSE(MainFrame::OnClose)
END_EVENT_TABLE()

MainFrame::MainFrame(const wxChar *title, int xpos, int ypos, int width, int height, std::string lp)
	: wxFrame (NULL, -1, title, wxPoint (xpos, ypos), wxSize (width, height)), lessonPath(lp)
{
    // first, the menu bar
    wxMenu *fileMenu = new wxMenu;
    wxMenu *viewMenu = new wxMenu;
    wxMenu *helpMenu = new wxMenu;
    wxMenuBar *menuBar = new wxMenuBar;
    
    fileMenu->Append (menuNew, "&New quiz", "Begin a new quiz");
    fileMenu->Append (menuRestart, "&Restart quiz", "Restarts current quiz from the beginning");
    fileMenu->AppendSeparator();
    fileMenu->Append (menuOptions, "&Options");
    fileMenu->AppendSeparator();
    fileMenu->Append (menuExit, "E&xit");
    viewMenu->Append (menuStats, "&Statistics", "Show your recorded progress and other statistics");
    helpMenu->Append (menuAbout, "&About");	
    menuBar->Append (fileMenu, "&File");
    menuBar->Append (viewMenu, "&View");
    menuBar->Append (helpMenu, "&Help");
    
    SetMenuBar (menuBar);
    CreateStatusBar (1);

    // now the controls
    wxBoxSizer *szrTop = new wxBoxSizer (wxVERTICAL);
    SetSizer (szrTop);
    
    wxBoxSizer *szrMiddle = new wxBoxSizer (wxHORIZONTAL);
    szrTop->Add (szrMiddle, 1, wxEXPAND|wxALL, 0);
    
    wxBoxSizer *szrMiddleLeft = new wxBoxSizer (wxVERTICAL);
    szrMiddle->Add (szrMiddleLeft, 1, wxEXPAND|wxALL, 0);
    
    lblResult1 = new wxStaticText (this, -1, "",
	wxDefaultPosition, wxDefaultSize);
    szrMiddleLeft->Add (lblResult1, 0, wxALIGN_CENTER, 0);
    wxFont f = lblResult1->GetFont();
    f.SetWeight (wxBOLD);
    f.SetPointSize (24);
    lblResult1->SetFont (f);
        
    lblResult2 = new wxStaticText (this, -1, "",
	wxDefaultPosition, wxDefaultSize);
    szrMiddleLeft->Add (lblResult2, 0, wxALIGN_CENTER, 0);
    
    txtOutput = new wxTextCtrl (this, -1, "", 
	wxDefaultPosition, wxSize (400, 200), wxTE_MULTILINE | wxTE_READONLY | wxTE_RICH); 
    wxTextAttr styleOutput;
    styleOutput.SetFont (wxFont(11, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
    txtOutput->SetDefaultStyle(styleOutput);
    szrMiddleLeft->Add (txtOutput, 1, wxEXPAND | wxALL, 0);        
    
    binPanel = new wxPanel (this, pnlPaint,
	wxDefaultPosition, wxSize (130, 200), wxSUNKEN_BORDER|wxFULL_REPAINT_ON_RESIZE);
    szrMiddle->Add (binPanel, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    
    txtInput = new wxTextCtrl (this, txtEnter, "", 
	wxDefaultPosition, wxSize (400, 20), wxTE_PROCESS_ENTER | wxTE_RICH);
    wxTextAttr styleInput;
    styleInput.SetFont (wxFont(11, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
    txtInput->SetDefaultStyle(styleInput);
    szrTop->Add (txtInput, 0, wxEXPAND | wxALL, 0);    
    txtInput->SetFocus();
    
    SetAutoLayout (true);
    Layout();
    
    //~ Connect (pnlPaint, wxEVT_PAINT, 
	//~ (wxObjectEventFunction)
	//~ (wxEventFunction)
	//~ (wxPaintEventFunction)&MainFrame::OnPanelPaint); // this syntax is copied from the "event.cpp" sample
	    // I don't understand it...
    binPanel->PushEventHandler (new PanelPainter(&quiz, binPanel));
    
    starttime = -1;
    startcounter = -1;

    init();    
}


MainFrame::~MainFrame()
{
}

void PanelPainter::OnPaint (wxPaintEvent &event)
{
  wxPaintDC dc (panel);
  
  dc.BeginDrawing();
  dc.SetBackground (wxBrush ("BLUE", wxSOLID));
  dc.Clear();
  
  if (quiz->IsValid())
  {
		const int barWidth = 100;
		const int barHeight = 20;
		const int marginTop = 10;
		const int marginLeft = 10;
		const int spaceBetween = 10;
		
		wxBrush solid("YELLOW", wxSOLID);
		wxBrush transparent("YELLOW", wxTRANSPARENT);
	    	
		dc.SetPen (wxPen ("YELLOW"));
		dc.SetBrush (solid);
		for (int i = 0; i < quiz->GetBins(); ++i)
		{
		    int pct = 100 * quiz->GetBinCount(i) / quiz->GetWordCount();
		    dc.DrawRectangle (marginLeft + barWidth - pct, 
		    	marginTop + i * (barHeight + spaceBetween), 
		    	pct, barHeight);   
		}
	    
		dc.SetPen (wxPen ("BLACK"));
		dc.SetBrush (transparent);
		for (int i = 0; i < quiz->GetBins(); ++i)
		{   
		    dc.DrawRectangle (marginLeft, 
		    	marginTop + i * (barHeight + spaceBetween), 
		    	barWidth, barHeight);   

		    dc.DrawText (wxString::Format("%i", quiz->GetBinCount(i)), 
				marginLeft + barWidth / 2,
				marginTop + i * (barHeight + spaceBetween));
		}
	}
  	dc.EndDrawing();
}

void MainFrame::DrawBins()
{
    binPanel->Refresh(); // repaint binPanel
}

void MainFrame::OnMenuNew (wxCommandEvent &event)
{
    selectLesson();
}

void MainFrame::OnMenuRestart (wxCommandEvent &event)
{
    if (quiz.IsValid())
    {
    	quiz.LoadLesson (quiz.GetFileName());
    	nextWord();
		writeLog(); // write progress to log
		// restart timer...
		beginLog();
    }
}

// restart timer
void MainFrame::beginLog ()
{
	startcounter = quiz.GetCounter();
	starttime = wxDateTime::Now();	
}
	
void MainFrame::writeLog ()
{
	wxDateTime endtime = wxDateTime::Now();
	wxTimeSpan span = endtime - starttime;
	wxString line = starttime.Format("%a %b %d %H:%M:%S %Y") +
		wxString::Format (" #q: %4i dur.: ", quiz.GetCounter() - startcounter) +
		span.Format ("%H:%M:%S") + 
		wxString::Format (" %s\n", quiz.GetFileName().c_str());
	if (quiz.GetCounter() - startcounter >= 10)
	{
		fstream f;
		
		f.open (LOGFILE, ios::out | ios::app);
		f << line.c_str();
		f.close ();
	}
	//wxMessageDialog *dlg = new wxMessageDialog(this, line);
	//dlg->ShowModal();
}
	
void MainFrame::OnMenuOptions (wxCommandEvent &event)
{
    if (quiz.IsValid())
	{
		OptionsDlg *dlg = new OptionsDlg (this);
		dlg->SetBins (quiz.GetBins());
		if (dlg->ShowModal() == wxID_OK)
		{
		    quiz.SetBins (dlg->GetBins());
		    DrawBins();
		}
    }
}

void MainFrame::OnMenuExit (wxCommandEvent &event)
{
    Close();
}

void MainFrame::OnMenuStats (wxCommandEvent &event)
{
    StatsDlg *dlg = new StatsDlg (this);
    dlg->LoadLogFile();    
    dlg->ShowModal();
}

void MainFrame::OnMenuAbout (wxCommandEvent &event)
{
    AboutDlg *dlg = new AboutDlg (this);
    dlg->ShowModal();
}

void MainFrame::OnTxtEnter (wxCommandEvent &event)
{
    if (quiz.IsValid() && !quiz.IsFinished())
    {
	checkAnswer();
	txtInput->SetValue("");

	if (quiz.IsFinished())
	{
	    writeLog(); // write progress to log
	    showResults();
	    clearFrame();
	}
	else
	{
	    nextWord();
	}
    }
}

void MainFrame::OnClose (wxCloseEvent &event)
{
    // save settings
    fstream f;
    f.open (SETTINGSFILE, ios::trunc | ios::out | ios::binary);
    quiz.Serialize (f);
	WriteString (f, lessonPath);
    f.close();
    
    writeLog(); // write progress to log
    
    Destroy(); // close app without further ado
}

// called at beginning, loads settings file and
// optionally loads stored quiz
void MainFrame::init()
{
    srand(time(0));
        
    fstream f;
    f.open (SETTINGSFILE, ios::in | ios::binary);
    if (f.good())
    {
        quiz.Deserialize (f);
		lessonPath = ReadString(f);
		if (!quiz.IsFinished())
        {
        	beginLog();
			nextWord();
		}
    }
    else
    {
		// lessonPath remains the default, leave untouched.
		clearFrame();
    }
    f.close();    
}

void MainFrame::selectLesson()
{
    wxFileDialog *dlg = new wxFileDialog (this, "Choose a lesson", lessonPath.c_str(), "",
        "Text files (*.txt)|*.txt|All files (*.*)|*.*", wxOPEN);
	
    if (dlg->ShowModal() == wxID_OK)
    {
		lessonPath = dlg->GetDirectory().c_str();
    	// stop logging current quiz, if it is there...
    	if (quiz.IsValid())
    	{
    		writeLog();
    	}
    	
        quiz.LoadLesson (dlg->GetPath().c_str());        
        if (quiz.IsValid())
        {   
        	beginLog();
            nextWord();
        }
        else
        {
            // fixme: should popup an error message dialog
            (*txtOutput) << "Error: could not load " << dlg->GetFilename() << "\n";
            (*txtOutput) << "Go to file->new to start a new lesson\n";
        }
    }    
}

// let Quiz pick next word, and display the question
void MainFrame::nextWord()
{
    quiz.NextQuestion();
    
    DrawBins();
    //~ for (int i = 0; i < quiz.GetBins(); ++i)
    //~ {
        //~ (*txtOutput) << "#" << i << "[ " << quiz.GetBinCount(i) << "] ";
    //~ }
    (*txtOutput) << "\n";
    (*txtOutput) << "Question #" << quiz.GetCounter() << ": " << quiz.GetQuestion().c_str() << "\n";
}

// run the answer through quiz to see if it is correct,
// and display the results. If the quiz is over, show the final results and clear the frame
void MainFrame::checkAnswer()
{
    if (!quiz.IsValid()) return;
    
    wxString myWxAnswer = txtInput->GetValue();
    wxRegEx ("\\^e").Replace (&myWxAnswer, "");    
    wxRegEx ("~n").Replace (&myWxAnswer, "");
    
    wxRegEx ("\"a").Replace (&myWxAnswer, "");        
    wxRegEx ("\"e").Replace (&myWxAnswer, "");
    wxRegEx ("\"i").Replace (&myWxAnswer, "");
    wxRegEx ("\"o").Replace (&myWxAnswer, "");    	    
    wxRegEx ("\"u").Replace (&myWxAnswer, "");
    
    wxRegEx ("\"A").Replace (&myWxAnswer, "");
    wxRegEx ("\"E").Replace (&myWxAnswer, "");
    wxRegEx ("\"I").Replace (&myWxAnswer, "");
    wxRegEx ("\"O").Replace (&myWxAnswer, "");
    wxRegEx ("\"U").Replace (&myWxAnswer, "");
    
    wxRegEx ("`a").Replace (&myWxAnswer, "");
    wxRegEx ("`e").Replace (&myWxAnswer, "");
    wxRegEx ("`i").Replace (&myWxAnswer, "");
    wxRegEx ("`o").Replace (&myWxAnswer, "");
    wxRegEx ("`u").Replace (&myWxAnswer, "");
    
    wxRegEx ("'a").Replace (&myWxAnswer, "");
    wxRegEx ("'e").Replace (&myWxAnswer, "");
    wxRegEx ("'i").Replace (&myWxAnswer, "");
    wxRegEx ("'o").Replace (&myWxAnswer, "");
    wxRegEx ("'u").Replace (&myWxAnswer, "");
    
    wxRegEx ("^\\?").Replace (&myWxAnswer, "");
    wxRegEx ("^!").Replace (&myWxAnswer, "");
	
    string myAnswer = myWxAnswer.c_str();
	
    txtOutput->SetValue ("");
	
    if (quiz.CompareAnswer (myAnswer))
    {
        lblResult1->SetLabel ("Correct!");
        lblResult1->SetForegroundColour (*wxGREEN);        
        (*txtOutput) << "The answer was :\"" << quiz.GetCorrectAnswer().c_str() << "\"\n";       
    }
    else
    {
        lblResult1->SetLabel ("Wrong!");
        lblResult1->SetForegroundColour (*wxRED);        
        (*txtOutput) << "You answered :\"" << myAnswer.c_str() << "\"\n";
        (*txtOutput) << "The answer was :\"" << quiz.GetCorrectAnswer().c_str() << "\"\n";       
    }
    
    if (quiz.HasHint())
    {
        (*txtOutput) << "Hint: " << quiz.GetHint().c_str() << "\n";
    }
}

// pop up the results dialog when the quiz is finished.
void MainFrame::showResults()
{
    ResultDlg *dlg = new ResultDlg (this);
	dlg->SetQuiz (&quiz);
    
    // pick the ones where quizCount is highest
    dlg->ShowModal();
}

// called whenever there is no question currently to display
void MainFrame::clearFrame()
{
    (*txtOutput) << "No current Lesson.\nGo to file->new to start a new lesson\n";
}
